@@ -3,6 +3,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdbool.h>
+#include <limits.h>
 #include <locale.h>
 #include <getopt.h>
 #include <signal.h>
@@ -77,6 +78,7 @@ print_usage(const char *prog_name)
         "  -m,--maximized                           start in maximized mode\n"
         "  -F,--fullscreen                          start in fullscreen mode\n"
         "  -L,--login-shell                         start shell as a login shell\n"
+        "  --pty=PATH                               display an existing PTY instead of creating one\n"
         "  -D,--working-directory=DIR               directory to start in (CWD)\n"
         "  -w,--window-size-pixels=WIDTHxHEIGHT     initial width and height, in pixels\n"
         "  -W,--window-size-chars=WIDTHxHEIGHT      initial width and height, in characters\n"
@@ -86,7 +88,7 @@ print_usage(const char *prog_name)
         "  -p,--print-pid=FILE|FD                   print PID to file or FD (only applicable in server mode)\n"
         "  -d,--log-level={info|warning|error|none} log level (warning)\n"
         "  -l,--log-colorize=[{never|always|auto}]  enable/disable colorization of log output on stderr\n"
-        "  -s,--log-no-syslog                       disable syslog logging (only applicable in server mode)\n"
+        "  -S,--log-no-syslog                       disable syslog logging (only applicable in server mode)\n"
         "  -v,--version                             show the version number and quit\n"
         "  -e                                       ignored (for compatibility with xterm -e)\n";
 
@@ -172,6 +174,10 @@ sanitize_signals(void)
         sigaction(i, &dfl, NULL);
 }
 
+enum {
+    PTY_OPTION = CHAR_MAX + 1,
+};
+
 int
 main(int argc, char *const *argv)
 {
@@ -209,6 +215,7 @@ main(int argc, char *const *argv)
         {"maximized",              no_argument,       NULL, 'm'},
         {"fullscreen",             no_argument,       NULL, 'F'},
         {"presentation-timings",   no_argument,       NULL, 'P'}, /* Undocumented */
+        {"pty",                    required_argument, NULL, PTY_OPTION},
         {"print-pid",              required_argument, NULL, 'p'},
         {"log-level",              required_argument, NULL, 'd'},
         {"log-colorize",           optional_argument, NULL, 'l'},
@@ -221,6 +228,7 @@ main(int argc, char *const *argv)
     bool check_config = false;
     const char *conf_path = NULL;
     const char *custom_cwd = NULL;
+    const char *pty_path = NULL;
     bool as_server = false;
     const char *conf_server_socket_path = NULL;
     bool presentation_timings = false;
@@ -253,7 +261,7 @@ main(int argc, char *const *argv)
             break;
 
         case 't':
-            tll_push_back(overrides, xasprintf("term=%s", optarg));
+            tll_push_back(overrides, xstrjoin("term=", optarg));
             break;
 
         case 'L':
@@ -261,11 +269,11 @@ main(int argc, char *const *argv)
             break;
 
         case 'T':
-            tll_push_back(overrides, xasprintf("title=%s", optarg));
+            tll_push_back(overrides, xstrjoin("title=", optarg));
             break;
 
         case 'a':
-            tll_push_back(overrides, xasprintf("app-id=%s", optarg));
+            tll_push_back(overrides, xstrjoin("app-id=", optarg));
             break;
 
         case 'D': {
@@ -279,7 +287,7 @@ main(int argc, char *const *argv)
         }
 
         case 'f': {
-            char *font_override = xasprintf("font=%s", optarg);
+            char *font_override = xstrjoin("font=", optarg);
             tll_push_back(overrides, font_override);
             break;
         }
@@ -316,6 +324,10 @@ main(int argc, char *const *argv)
                 conf_server_socket_path = optarg;
             break;
 
+        case PTY_OPTION:
+            pty_path = optarg;
+            break;
+
         case 'P':
             presentation_timings = true;
             break;
@@ -351,11 +363,11 @@ main(int argc, char *const *argv)
         }
 
         case 'l':
-            if (optarg == NULL || strcmp(optarg, "auto") == 0)
+            if (optarg == NULL || streq(optarg, "auto"))
                 log_colorize = LOG_COLORIZE_AUTO;
-            else if (strcmp(optarg, "never") == 0)
+            else if (streq(optarg, "never"))
                 log_colorize = LOG_COLORIZE_NEVER;
-            else if (strcmp(optarg, "always") == 0)
+            else if (streq(optarg, "always"))
                 log_colorize = LOG_COLORIZE_ALWAYS;
             else {
                 fprintf(stderr, "%s: argument must be one of 'never', 'always' or 'auto'\n", optarg);
@@ -383,6 +395,11 @@ main(int argc, char *const *argv)
         }
     }
 
+    if (as_server && pty_path) {
+        fputs("error: --pty is incompatible with server mode\n", stderr);
+        return ret;
+    }
+
     log_init(log_colorize, as_server && log_syslog,
              as_server ? LOG_FACILITY_DAEMON : LOG_FACILITY_USER, log_level);
 
@@ -427,7 +444,7 @@ main(int argc, char *const *argv)
 
         /*
          * Try to force an UTF-8 locale. If we succeed, launch the
-         * user’s shell as usual, but add a user-notification saying
+         * user's shell as usual, but add a user-notification saying
          * the locale has been changed.
          */
         for (size_t i = 0; i < ALEN(fallback_locales); i++) {
@@ -538,7 +555,7 @@ main(int argc, char *const *argv)
 
         if (resolved_path_cwd != NULL &&
             resolved_path_pwd != NULL &&
-            strcmp(resolved_path_cwd, resolved_path_pwd) == 0)
+            streq(resolved_path_cwd, resolved_path_pwd))
         {
             /*
              * The resolved path of $PWD matches the resolved path of
@@ -574,7 +591,7 @@ main(int argc, char *const *argv)
         goto out;
 
     if (!as_server && (term = term_init(
-                           &conf, fdm, reaper, wayl, "foot", cwd, token,
+                           &conf, fdm, reaper, wayl, "foot", cwd, token, pty_path,
                            argc, argv, NULL,
                            &term_shutdown_cb, &shutdown_ctx)) == NULL) {
         goto out;
