@@ -121,8 +121,11 @@ struct row {
     bool dirty;
     bool linebreak;
 
-    /* Shell integration */
-    bool prompt_marker;
+    struct {
+        bool prompt_marker;
+        int cmd_start;  /* Column, -1 if unset */
+        int cmd_end;    /* Column, -1 if unset */
+    } shell_integration;
 };
 
 struct sixel {
@@ -149,8 +152,8 @@ struct sixel {
      * We store the cell dimensions of the time the sixel was emitted.
      *
      * If the font size is changed, we rescale the image accordingly,
-     * to ensure it stays within its cell boundaries. ‘scaled’ is a
-     * cached, rescaled version of ‘data’ + ‘pix’.
+     * to ensure it stays within its cell boundaries. 'scaled' is a
+     * cached, rescaled version of 'data' + 'pix'.
      */
     int cell_width;
     int cell_height;
@@ -300,6 +303,7 @@ enum selection_kind {
     SELECTION_NONE,
     SELECTION_CHAR_WISE,
     SELECTION_WORD_WISE,
+    SELECTION_QUOTE_WISE,
     SELECTION_LINE_WISE,
     SELECTION_BLOCK
 };
@@ -342,7 +346,7 @@ struct url {
     char32_t *key;
     struct range range;
     enum url_action action;
-    bool url_mode_dont_change_url_attr; /* Entering/exiting URL mode doesn’t touch the cells’ attr.url */
+    bool url_mode_dont_change_url_attr; /* Entering/exiting URL mode doesn't touch the cells' attr.url */
     bool osc8;
     bool duplicate;
 };
@@ -378,7 +382,7 @@ struct terminal {
     bool bracketed_paste;
     bool focus_events;
     bool alt_scrolling;
-    bool modify_other_keys_2;  /* True when modifyOtherKeys=2 (i.e. “CSI >4;2m”) */
+    bool modify_other_keys_2;  /* True when modifyOtherKeys=2 (i.e. "CSI >4;2m") */
     enum cursor_origin origin;
     enum cursor_keys cursor_keys_mode;
     enum keypad_keys keypad_keys_mode;
@@ -405,6 +409,7 @@ struct terminal {
     bool font_is_sized_by_dpi;
     int16_t font_x_ofs;
     int16_t font_y_ofs;
+    int16_t font_baseline;
     enum fcft_subpixel font_subpixel;
 
     struct {
@@ -468,6 +473,7 @@ struct terminal {
         bool alt_screen:1;
         bool ime:1;
         bool app_sync_updates:1;
+        bool grapheme_shaping:1;
 
         bool sixel_display_mode:1;
         bool sixel_private_palette:1;
@@ -477,6 +483,7 @@ struct terminal {
     bool window_title_has_been_set;
     char *window_title;
     tll(char *) window_title_stack;
+    char *app_id;
 
     struct {
         bool active;
@@ -489,6 +496,7 @@ struct terminal {
     } blink;
 
     float scale;
+    float scale_before_unmap;  /* Last scaling factor used */
     int width;  /* pixels */
     int height; /* pixels */
     int stashed_width;
@@ -596,10 +604,14 @@ struct terminal {
 
         struct {
             struct timespec last_update;
-            bool is_armed;
             int timer_fd;
         } title;
 
+        struct {
+            struct timespec last_update;
+            int timer_fd;
+        } app_id;
+
         uint32_t scrollback_lines; /* Number of scrollback lines, from conf (TODO: move out from render struct?) */
 
         struct {
@@ -637,7 +649,7 @@ struct terminal {
     } render;
 
     struct {
-        struct grid *grid;    /* Original ‘normal’ grid, before resize started */
+        struct grid *grid;    /* Original 'normal' grid, before resize started */
         int old_screen_rows;  /* term->rows before resize started */
         int old_cols;         /* term->cols before resize started */
         int old_hide_cursor;  /* term->hide_cursor before resize started */
@@ -654,7 +666,6 @@ struct terminal {
         } state;
 
         struct coord pos;    /* Current sixel coordinate */
-        size_t row_byte_ofs; /* Byte position into image, for current row */
         int color_idx;       /* Current palette index */
         uint32_t *private_palette;   /* Private palette, used when private mode 1070 is enabled */
         uint32_t *shared_palette;    /* Shared palette, used when private mode 1070 is disabled */
@@ -663,15 +674,18 @@ struct terminal {
 
         struct {
             uint32_t *data;  /* Raw image data, in ARGB */
+            uint32_t *p;     /* Pointer into data, for current position */
             int width;       /* Image width, in pixels */
             int height;      /* Image height, in pixels */
+            int alloc_height;
+            unsigned int bottom_pixel;
         } image;
 
         /*
          * Pan is the vertical shape of a pixel
          * Pad is the horizontal shape of a pixel
          *
-         * pan/pad is the sixel’s aspect ratio
+         * pan/pad is the sixel's aspect ratio
          */
         int pan;
         int pad;
@@ -716,13 +730,16 @@ struct terminal {
 
     char *foot_exe;
     char *cwd;
+
+    bool grapheme_shaping;
 };
 
 struct config;
 struct terminal *term_init(
     const struct config *conf, struct fdm *fdm, struct reaper *reaper,
     struct wayland *wayl, const char *foot_exe, const char *cwd,
-    const char *token, int argc, char *const *argv, char *const *envp,
+    const char *token, const char *pty_path,
+    int argc, char *const *argv, const char *const *envp,
     void (*shutdown_cb)(void *data, int exit_code), void *shutdown_data);
 
 bool term_shutdown(struct terminal *term);
@@ -737,6 +754,7 @@ bool term_paste_data_to_slave(
     struct terminal *term, const void *data, size_t len);
 
 bool term_fractional_scaling(const struct terminal *term);
+bool term_preferred_buffer_scale(const struct terminal *term);
 bool term_update_scale(struct terminal *term);
 bool term_font_size_increase(struct terminal *term);
 bool term_font_size_decrease(struct terminal *term);
@@ -783,6 +801,8 @@ void term_cursor_down(struct terminal *term, int count);
 void term_cursor_blink_update(struct terminal *term);
 
 void term_print(struct terminal *term, char32_t wc, int width);
+void term_fill(struct terminal *term, int row, int col, uint8_t c, size_t count,
+               bool use_sgr_attrs);
 
 void term_scroll(struct terminal *term, int rows);
 void term_scroll_reverse(struct terminal *term, int rows);
@@ -823,6 +843,7 @@ void term_xcursor_update_for_seat(struct terminal *term, struct seat *seat);
 void term_set_user_mouse_cursor(struct terminal *term, const char *cursor);
 
 void term_set_window_title(struct terminal *term, const char *title);
+void term_set_app_id(struct terminal *term, const char *app_id);
 void term_flash(struct terminal *term, unsigned duration_ms);
 void term_bell(struct terminal *term);
 bool term_spawn_new(const struct terminal *term);
@@ -837,6 +858,8 @@ bool term_scrollback_to_text(
     const struct terminal *term, char **text, size_t *len);
 bool term_view_to_text(
     const struct terminal *term, char **text, size_t *len);
+bool term_command_output_to_text(
+    const struct terminal *term, char **text, size_t *len);
 
 bool term_ime_is_enabled(const struct terminal *term);
 void term_ime_enable(struct terminal *term);
