@@ -60,7 +60,7 @@ test_string(struct context *ctx, bool (*parse_fun)(struct context *ctx),
                 BUG("[%s].%s=%s: failed to parse",
                     ctx->section, ctx->key, ctx->value);
             }
-            if (strcmp(*ptr, input[i].value) != 0) {
+            if (!streq(*ptr, input[i].value)) {
                 BUG("[%s].%s=%s: set value (%s) not the expected one (%s)",
                     ctx->section, ctx->key, ctx->value,
                     *ptr, input[i].value);
@@ -357,9 +357,7 @@ test_spawn_template(struct context *ctx, bool (*parse_fun)(struct context *ctx),
         BUG("[%s].%s=%s: argv is NULL", ctx->section, ctx->key, ctx->value);
 
     for (size_t i = 0; i < ALEN(args); i++) {
-        if (ptr->argv.args[i] == NULL ||
-            strcmp(ptr->argv.args[i], args[i]) != 0)
-        {
+        if (ptr->argv.args[i] == NULL || !streq(ptr->argv.args[i], args[i])) {
             BUG("[%s].%s=%s: set value not the expected one: "
                 "mismatch of arg #%zu: expected=\"%s\", got=\"%s\"",
                 ctx->section, ctx->key, ctx->value, i,
@@ -426,6 +424,10 @@ test_color(struct context *ctx, bool (*parse_fun)(struct context *ctx),
         {"ffffff", 0xffffff},
         {"ffffffff", 0xffffffff, !alpha_allowed},
         {"aabbccdd", 0xaabbccdd, !alpha_allowed},
+        {"00", 0, true},
+        {"0000", 0, true},
+        {"00000", 0, true},
+        {"000000000", 0, true},
         {"unittest-invalid-color", 0, true},
     };
 
@@ -785,6 +787,17 @@ test_section_csd(void)
     config_free(&conf);
 }
 
+static bool
+have_modifier(const config_modifier_list_t *mods, const char *mod)
+{
+    tll_foreach(*mods, it) {
+        if (strcmp(it->item, mod) == 0)
+            return true;
+    }
+
+    return false;
+}
+
 static void
 test_key_binding(struct context *ctx, bool (*parse_fun)(struct context *ctx),
                  int action, int max_action, const char *const *map,
@@ -875,7 +888,7 @@ test_key_binding(struct context *ctx, bool (*parse_fun)(struct context *ctx),
 
         for (size_t i = 0; i < ALEN(args); i++) {
             if (binding->aux.pipe.args[i] == NULL ||
-                strcmp(binding->aux.pipe.args[i], args[i]) != 0)
+                !streq(binding->aux.pipe.args[i], args[i]))
             {
                 BUG("[%s].%s=%s: pipe argv not the expected one: "
                     "mismatch of arg #%zu: expected=\"%s\", got=\"%s\"",
@@ -902,17 +915,19 @@ test_key_binding(struct context *ctx, bool (*parse_fun)(struct context *ctx),
             ctx->section, ctx->key, ctx->value, binding->action, action);
     }
 
-    if (binding->modifiers.ctrl != ctrl ||
-        binding->modifiers.alt != alt ||
-        binding->modifiers.shift != shift ||
-        binding->modifiers.super != super)
+    bool have_ctrl = have_modifier(&binding->modifiers, XKB_MOD_NAME_CTRL);
+    bool have_alt = have_modifier(&binding->modifiers, XKB_MOD_NAME_ALT);
+    bool have_shift = have_modifier(&binding->modifiers, XKB_MOD_NAME_SHIFT);
+    bool have_super = have_modifier(&binding->modifiers, XKB_MOD_NAME_LOGO);
+
+    if (have_ctrl != ctrl || have_alt != alt ||
+        have_shift != shift || have_super != super)
     {
         BUG("[%s].%s=%s: modifier mismatch:\n"
             "  have:     ctrl=%d, alt=%d, shift=%d, super=%d\n"
             "  expected: ctrl=%d, alt=%d, shift=%d, super=%d",
             ctx->section, ctx->key, ctx->value,
-            binding->modifiers.ctrl, binding->modifiers.alt,
-            binding->modifiers.shift, binding->modifiers.super,
+            have_ctrl, have_alt, have_shift, have_super,
             ctrl, alt, shift, super);
     }
 
@@ -968,14 +983,17 @@ _test_binding_collisions(struct context *ctx,
     bindings.arr[0] = (struct config_key_binding){
         .action = (test_mode == FAIL_DIFFERENT_ACTION
                    ? max_action - 1 : max_action),
-        .modifiers = {.ctrl = true},
+        .modifiers = tll_init(),
         .path = "unittest",
     };
+    tll_push_back(bindings.arr[0].modifiers, xstrdup(XKB_MOD_NAME_CTRL));
+
     bindings.arr[1] = (struct config_key_binding){
         .action = max_action,
-        .modifiers = {.ctrl = true},
+        .modifiers = tll_init(),
         .path = "unittest",
     };
+    tll_push_back(bindings.arr[1].modifiers, xstrdup(XKB_MOD_NAME_CTRL));
 
     switch (type) {
     case KEY_BINDING:
@@ -996,7 +1014,8 @@ _test_binding_collisions(struct context *ctx,
         break;
 
     case FAIL_MOUSE_OVERRIDE:
-        ctx->conf->mouse.selection_override_modifiers.ctrl = true;
+        tll_free_and_free(ctx->conf->mouse.selection_override_modifiers, free);
+        tll_push_back(ctx->conf->mouse.selection_override_modifiers, xstrdup(XKB_MOD_NAME_CTRL));
         break;
 
     case FAIL_DIFFERENT_ARGV:
@@ -1235,10 +1254,13 @@ test_section_text_bindings(void)
     ctx.key = "\\y";
     xassert(!parse_section_text_bindings(&ctx));
 
+#if 0
+    /* Invalid modifier and key names are detected later, when a
+     * layout is applied */
     ctx.key = "abcd";
     ctx.value = "InvalidMod+y";
     xassert(!parse_section_text_bindings(&ctx));
-
+#endif
     config_free(&conf);
 }
 
@@ -1254,26 +1276,26 @@ test_section_environment(void)
     ctx.value = "bar";
     xassert(parse_section_environment(&ctx));
     xassert(tll_length(conf.env_vars) == 1);
-    xassert(strcmp(tll_front(conf.env_vars).name, "FOO") == 0);
-    xassert(strcmp(tll_front(conf.env_vars).value, "bar") == 0);
+    xassert(streq(tll_front(conf.env_vars).name, "FOO"));
+    xassert(streq(tll_front(conf.env_vars).value, "bar"));
 
     /* Add a second variable */
     ctx.key = "BAR";
     ctx.value = "123";
     xassert(parse_section_environment(&ctx));
     xassert(tll_length(conf.env_vars) == 2);
-    xassert(strcmp(tll_back(conf.env_vars).name, "BAR") == 0);
-    xassert(strcmp(tll_back(conf.env_vars).value, "123") == 0);
+    xassert(streq(tll_back(conf.env_vars).name, "BAR"));
+    xassert(streq(tll_back(conf.env_vars).value, "123"));
 
     /* Replace the *value* of the first variable */
     ctx.key = "FOO";
     ctx.value = "456";
     xassert(parse_section_environment(&ctx));
     xassert(tll_length(conf.env_vars) == 2);
-    xassert(strcmp(tll_front(conf.env_vars).name, "FOO") == 0);
-    xassert(strcmp(tll_front(conf.env_vars).value, "456") == 0);
-    xassert(strcmp(tll_back(conf.env_vars).name, "BAR") == 0);
-    xassert(strcmp(tll_back(conf.env_vars).value, "123") == 0);
+    xassert(streq(tll_front(conf.env_vars).name, "FOO"));
+    xassert(streq(tll_front(conf.env_vars).value, "456"));
+    xassert(streq(tll_back(conf.env_vars).name, "BAR"));
+    xassert(streq(tll_back(conf.env_vars).value, "123"));
 
     config_free(&conf);
 }
